/* $Id: LADSPAPluginDescriptor.cc,v 1.1 2003/08/07 08:57:08 andete Exp $

lemux - LADSPA instrument plugins based on openMSX devices

(c) 2003 Joost Yervante Damad <joost@lumatec.be>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "LADSPAPluginDescriptor.hh"
#include "LADSPAPortDescriptor.hh"
#include "LADSPAPluginInstance.hh"

static LADSPA_Handle instantiate_wrapper(const LADSPA_Descriptor* Descriptor, unsigned long sampleRate);
static void connect_port_wrapper(LADSPA_Handle Instance, unsigned long Port, LADSPA_Data* DataLocation);
static void activate_wrapper(LADSPA_Handle Instance);
static void run_wrapper(LADSPA_Handle Instance, unsigned long SampleCount);
static void cleanup_wrapper(LADSPA_Handle Instance);

const LADSPA_Descriptor* LADSPAPluginDescriptor::getLADSPA_Descriptor()
{
	syncPorts();
	return this;
}

LADSPAPluginDescriptor::LADSPAPluginDescriptor(unsigned long uniqueID, const std::string& label, const std::string& name, const std::string& maker, const std::string& copyright, LADSPA_Properties properties, void* implementationData)
:_label(label), _name(name), _maker(maker), _copyright(copyright)
{
	UniqueID           = uniqueID;
	Label              = _label.c_str();
	Name               = _name.c_str();
	Maker              = _maker.c_str();
	Copyright          = _copyright.c_str();
	Properties         = properties;
	ImplementationData = implementationData;
	PortCount          = 0;
	PortNames          = 0;
	PortDescriptors    = 0;
	PortRangeHints     = 0;

	// callback methods:
	instantiate        = instantiate_wrapper;
	connect_port       = connect_port_wrapper;
	activate           = activate_wrapper;
	run_adding         = 0;
	set_run_adding_gain= 0;
	deactivate         = 0;
	cleanup            = cleanup_wrapper;
	run                = run_wrapper;

}

LADSPAPluginDescriptor::~LADSPAPluginDescriptor()
{
	for (std::vector<LADSPAPortDescriptor*>::iterator i = m_ports.begin(); i != m_ports.end(); ++i)
	{
		delete (*i);
	}
}

void LADSPAPluginDescriptor::addPort(LADSPAPortDescriptor* port)
{
	m_ports.push_back(port);
}

void LADSPAPluginDescriptor::syncPorts()
{
	if (PortNames != 0)
	{
		delete[] PortNames;
		delete[] PortDescriptors;
		delete[] PortRangeHints;
		PortNames = 0;
		PortDescriptors = 0;
		PortRangeHints = 0;
	}
	PortCount = m_ports.size();
	if (PortCount > 0)
	{
		const char** pn = new const char*[PortCount];
		PortNames = pn;
		LADSPA_PortDescriptor* pd = new LADSPA_PortDescriptor[PortCount];
		PortDescriptors = pd;
		LADSPA_PortRangeHint* pr = new LADSPA_PortRangeHint[PortCount];
		PortRangeHints = pr;
		int j = 0;
		for (std::vector<LADSPAPortDescriptor*>::const_iterator i = m_ports.begin(); i!= m_ports.end(); ++i, ++j)
		{
			pn[j] = (*i)->getName().c_str();
			pd[j] = (*i)->getType();
			pr[j] = (*i)->getRangeHint();
		}
	}
}

static LADSPA_Handle instantiate_wrapper(const LADSPA_Descriptor* Descriptor, unsigned long sampleRate)
{
	LADSPAPluginDescriptor* d = (LADSPAPluginDescriptor*)Descriptor;
	return d->create(sampleRate);
}

static void connect_port_wrapper(LADSPA_Handle Instance, unsigned long port, LADSPA_Data* dataLocation)
{
	LADSPAPluginInstance* i = (LADSPAPluginInstance*)Instance;
	i->connect_port(port, dataLocation);
}

static void activate_wrapper(LADSPA_Handle Instance)
{
	LADSPAPluginInstance* i = (LADSPAPluginInstance*)Instance;
	i->activate();
}

static void run_wrapper(LADSPA_Handle Instance, unsigned long SampleCount)
{
	LADSPAPluginInstance* i = (LADSPAPluginInstance*)Instance;
	i->run(SampleCount);
}

static void cleanup_wrapper(LADSPA_Handle Instance)
{
	LADSPAPluginInstance* i = (LADSPAPluginInstance*)Instance;
	delete i;
}

